/*
  Compile:

  gcc -g -Wall -O2 -o aftertest after.c -lm \
      -D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE  \
      -DHAVE_LIBNETCDF -I/client/include -L/client/lib -lnetcdf \
      -DHAVE_LIBPTHREAD -lpthread
*/

#ifndef _AFTERDOC_H
#define _AFTERDOC_H
#endif

/* ============================================================= */
/*                                                               */
/* postprocessing program for ECHAM data and ECMWF analysis data */
/*                                                               */
/* Luis     Kornblueh   - MPI    Hamburg                         */
/* Uwe      Schulzweida - MPI    Hamburg                         */
/* Arno     Hellbach    - DKRZ   Hamburg                         */
/* Edilbert Kirk        - MI Uni Hamburg                         */
/* Michael  Ponater     - DLR    Oberpfaffenhofen                */
/*                                                               */
/* ============================================================= */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* REVISION HISTORY                                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.11 (Arno - 25-Aug-94)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* -> included processing of GRIB Version 1                      */
/*    neccessary changes:                                        */
/*    - defined global variable GribEdition                      */
/*    - changed readgrib to read full GRIB message either 1 or 0 */
/*    - changed decogb in order to skip 4 bytes after            */
/*      GRIB block 0 and before GRIB block 2 if GribEdition == 1 */
/*                                                               */
/* -> included timesel to hold up to 4 times                     */
/*    neccessary changes:                                        */
/*    - removed global variable timesel,                         */
/*      timesel as input parameter name is still valid !!        */
/*    - defined global constant MAX_HOURS ( = 4)                 */
/*    - defined global field hours[MAX_HOURS]                    */
/*    - defined global variable nrqh as a counter                */
/*    - added routine scantime (adapted from scanlevel)          */
/*    - changed AnalysisProcess and EchamProcess to enable       */
/*      selection of more than one time                          */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.12 (Edilbert - 29-Aug-94)                           */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* -> humidity is now optional in model data sets:               */
/*    MakeGeopotHeight() checks if humidity is present and uses  */
/*    either a wet formula (with virtual temperature) or a dry   */
/*    one. Missing humidity in model data is not a fatal error.  */
/*                                                               */
/* -> performance optimization:                                  */
/*    Functions PutnByte() replaced my macros                    */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.2 (Edilbert - 04-Jul-95)                            */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* -> The FORTRAN FFT-subroutines are replaced by C-functions    */
/*    Now this is a pure C-program - the new FFT routines are    */
/*    slightly faster on the CRAY than the old ones.             */
/*                                                               */
/* -> The afterburner can now process grib files from models     */
/*    which simulate the atmosphere of planet Mars               */
/*    namelist parameter: mars=1                                 */
/*                                                               */
/* -> The Free function now returns a NULL which can be used     */
/*    to clear the pointer, that was freed.                      */
/*                                                               */
/* -> The number of latitudes is calculated from truncation      */
/*    with Latitudes = 2 * ((Truncation * 3 + 3) / 4)            */
/*    to allow all resolutions                                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.21 (Arno - 24-Aug-95)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> used model ID (byte 5 of block 1) to determine wether NCAR */
/*    based atmospheric model is used (retrieved in precntl)     */
/*    model ID's known at DKRZ (so far):                         */
/*                2 HOPE Ocean Model                             */
/*                3 OPYC Ocean Model                             */
/*                4  LSG Ocean Model                             */
/*               21 NCAR based AGCM                              */
/*               31 ECHAM 1/2/3                                  */
/*               50 ECHAM 4                                      */
/*              121 ECMWF Analysis                               */
/*                                                               */
/* -> provided code to decode complex packed spherical harmonics */
/*    changed decogb (added one if branch to be able to get info */
/*    variable PackComplexStart and to get PackComplexScale,     */
/*    indicator is variable PackComplex = TRUE/FALSE)            */
/*    added functions ScaleComplex and ScatterComplex called by  */
/*    *Control functions.                                        */
/*                                                               */
/* -> added code to calculate layer integrals of cloud cover and */
/*    liquid water content (LayerCloud and LayerWater).          */
/*    New codes:                                                 */
/*               34 integral of low cloud cover 750 - 1013 hPa   */
/*               35 integral of mid cloud cover 460 -  730 hPa   */
/*               36 integral of hih cloud cover  50 -  440 hPa   */
/*               37 integral of low LWC         750 - 1013 hPa   */
/*               38 integral of mid LWC         460 -  730 hPa   */
/*               39 integral of hih LWC          50 -  440 hPa   */
/*               40 integral of all LWC          50 - 1013 hPa   */
/*                                                               */
/* -> added function statistics                                  */
/*    statistics prints time and memory counts performed in main */
/*    by calls to clock() and in the *Alloc functions, by calls  */
/*    to mallinfo() to get the number of used blocks             */
/*                                                               */
/* -> changed data type of level[] to double                     */
/*    neccessary to process level below 1 Pa, i.e. for ECHAM/L43 */
/*    or more.                                                   */
/*                                                               */
/* -> changed all occurences of half_press to HalfPress->hybrid  */
/*    since both were the same in the old versions (minimizes    */
/*    memory allocation)                                         */
/*                                                               */
/* -> performance optimization:                                  */
/*                                                               */
/*    included #pragma _CRI inline statements for ExtraT/ExtraZ  */
/*    to force inlining of those functions in Interpolate_T/Z.   */
/*                                                               */
/*    rewrote encoding according to ECHAM4 procedure (E.Kirk).   */
/*    provided code for machines which do not have _pack routine */
/*                                                               */
/*    rewrote decoding according to encoding (used _unpack)      */
/*                                                               */
/*    changed legini in order to read polinomial coefficients    */
/*    instead of calculating them (done for T21/30/42/63/106).   */
/*    if FILE *legpol == NULL, calculation is done as usual.     */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.4 (Edilbert - 29-Nov-95)                            */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* -> on SUN and IBM the non-GRIB output file is written in      */
/*    FORTRAN binary form (with controlwords just before and     */
/*    after the data records. (#define FORTRAN_OUTPUT)           */
/*                                                               */
/* -> After ECHAM-GRIB and ECMWF-GRIB a third data format is     */
/*    recognized in the input file:                              */
/*    PUMA (Portable University Model of the Atmosphere) files   */
/*    are processed like ECHAM files                             */
/*                                                               */
/* -> Introduced command line options                            */
/*    -c for list of codes                                       */
/*    -d for additional debug output (previous Debug parameter)  */
/*    -h help (listing of command line syntax)                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.5 (Arno - 16-Jul-96)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> Changed algorithm for saturation vapour pressure in sh2rh  */
/*    and rh2sh according to ECMWF IFS code, since the reanalysis*/
/*    project has been performed using this code.                */
/*                                                               */
/* -> The reanalysis data comes with 2 new packing types:        */
/*    - complex packed spherical harmonics                       */
/*    - data on quasi-regular (reduced) grids                    */
/*                                                               */
/*    In order to come up with these function decogb was modified*/
/*    mainly in section 2 and 4. 2 global flags were introduced: */
/*    PackComplex and ReducedGrid. There are 3 new functions     */
/*    named ScatterComplex, ScaleComplex and qu2reg2_, the latter*/
/*    one in a separate source file.                             */
/*                                                               */
/* -> Introduced command line option                             */
/*    -s to reduce memory requirements.                          */
/*       Method: swap out (and Free) 3d grid fields and          */
/*               swap them in (and Alloc).                       */
/*                                                               */
/*    3 new functions are invoked: swapini, swapout and swapin.  */
/*    swapini scans environment for TMPDIR and constructs        */
/*            swapname[codeno]                                   */
/*    swapout opens swapfile[codeno], writes 3d grid field and   */
/*            closes swapfile[codeno]. Always used in conjunction*/
/*            with Free.                                         */
/*    swapin  opens swapfile[codeno], reads  3d grid field and   */
/*            closes swapfile[codeno]. Always used in conjunction*/
/*            with DoubleAlloc.                                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.6 (Arno - 16-Feb-97)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> Changed GRIB read algorithm in order to minimize the fread */
/*    library calls. The idea was to read blocks as large as the */
/*    environment allows. Therefore the token MAX_RBUF_LENGTH is */
/*    defined. The blocks (sizes of multiples of timesteps) are  */
/*    then scanned for "GRIB" strings and a pointer is returned  */
/*    to the calling routine. This minimizes system CPU time     */
/*    extraordinarily.                                           */
/*                                                               */
/* -> In order to make the afterburner pipe-able the "rewind"    */
/*    had to be erased. Due to this the PUMA processing couldn't */
/*    be hold longer inside the afterburner. I apologize.        */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.64 (Arno - 22-Jul-97)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> Invited "level error-handling"                             */
/*    Some ugly combinations of double typed and non existent    */
/*    levels caused dependent routines like rh2sh() to crash,    */
/*    because 2D-slices were used as initialised (that is, 0).   */
/*    The number and location of "levels" in GRIB-input is       */
/*    determined in function precntl(). In scanlevel(), this     */
/*    list is compared with the "requested levels" and the       */
/*    "requested levels" themselves are checked for redundancy.  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.70 (Arno - 15-Oct-97)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> saturation water vapour pressure formula adopted from IFS  */
/*    (the Integrated Forecast System, ECMWF) was changed in a   */
/*    way that the ice phase is now neglected and even below 0 C */
/*    the pressure is calculated assuming conditions over water. */
/*                                                               */
/* -> bufferlength RBufLength can now be passed as a line argu-  */
/*    ment with "-b". Per default RBufLength = 13 MB which is    */
/*    app. the length of a single-date T106-output. This can be  */
/*    reduced (and therefore memory) if coarser resolutions are  */
/*    to be processed.                                           */
/*                                                               */
/* -> PUMA is in again and also pipe-able now.                   */
/*                                                               */
/* -> An inventory is made in the pre-processing stage. One time-*/
/*    step is scanned and a list of codes available is made.     */
/*    The list is used to determine if any code "selected" shall */
/*    be calculated as a derived code or just decoded.           */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 2.80 (Edilbert - 21-Oct-97)                           */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> The PUMA file format was changed to IEEE                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.00 (Uwe - 12-Jan-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> all INT to int                                             */
/*    all REAL to double                                         */
/*    lgrib to unsigned int                                      */
/*    FileLength,SaveRBufLength and RBufLength to size_t         */
/*    convert to ANSI C                                          */
/*                                                               */
/* -> runs correct with 32 bit integer now                       */
/*      (fix bug with ibits == 32)                               */
/*                                                               */
/* -> performance optimization:                                  */
/*                                                               */
/*      included #pragma loop scalar for vpp7                    */
/*      included #pragma vdir for nec                            */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.01 (Luis - 09-Feb-99)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> replaced gauaw, which failed to converge for truncations   */
/*    larger than T213, and added float.h.                       */
/*                                                               */
/* -> There seems to be some reorgnising of the code necessary.  */
/*    Up to now the whole bunch of data is loaded into memory.   */
/*    For the newest ECMWF analysis this is getting to big ...   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.02 (Uwe - 12-Feb-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> reorganize the Buffer handling                             */
/*    (main, AnalysisControl, EchamControl, precntl)             */
/*                                                               */
/* -> performance optimization for NEC by Eckhard Tschirschnitz  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.03 (Uwe - 18-Feb-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> use mallinfo if malloc.h is included                       */
/*    use mallinfo for all memory allocation                     */
/* -> FORTRAN_OUTPUT if CRAY is undefinded                       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.04 (Andreas Rhodin 22-Feb-99)                       */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> introduced parameter LAT                                   */
/*    purpose: specify number of latitudes in gridpoint space    */
/*             independently from spectral truncation.           */
/*    works only if input is spherical harmonics,                */
/*    otherwise aborts with 'Latitude/Longitude Conflict'.       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.05 (Uwe - 23-Feb-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> MakeGeopotHeight: split loops for save vectorization       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.06 (Uwe - 03-Mar-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> new function gp2sp                                         */
/* -> use gp2sp if spectral Divergence,Vorticity or LnPs missing */
/* -> use MaximalProductLength for RBuf                          */
/* -> scancode: code=-1 means all detected codes selected        */
/* -> scanlevel: level=-1 means all detected levels selected     */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.07 (Uwe - 09-Mar-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> GribFromBuffer: use ItemsInBufr for RBuf                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.08 (Uwe - 18-Mar-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> use RBufCount in CheckAnalyses and CheckContent            */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.09 (Uwe+Luis - 19-Apr-99)                           */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> output GRIB version 1                                      */
/* -> set MIN_RBUF_LENGTH to 5MB                                 */
/* -> add leveltype and layer to struct Control                  */
/*    use layer for GRIB output for leveltype LEV_DOWN and       */
/*    LEV_HEIGHT                                                 */
/* -> use TimeU, Time1, T_R_I and Average for GRIB output        */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.10 (Uwe - 16-Sep-99)                                */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> Calculate MaximalProductLength after read all codes        */
/* -> option -a    : forces analysis data process                */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.11 (Uwe - 18-Feb-2000)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change (nh > Dim3GP) to (nh >= Dim3GP)  (Edi Kirk)         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.20 (Uwe - 30-Mar-2000)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> remove correction of incomming spectral fields             */
/* -> add 0.5 to the packed GRIB outputfile to get EMOS          */
/*    compatible results                                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.21 (Uwe - 22-May-2000)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> use HHMM instead of only HH in time record for service     */
/*        output                                                 */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.22 (Uwe - 21-Feb-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> T341 support                                               */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.40 (Uwe - 21-April-2001)                            */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> split after.c in:                                          */
/*      afterprog.c alloc.c fft.c     interpolation.c  physc.c   */
/*      convert.c   grib.c  service.c util.c           puma.c    */
/* -> Ansi C conform source files:                               */
/*      remove:   RegularFile, FIFO, SymbolicLink                */
/* -> Swap does not work with Mean:                              */
/*      exit if Swap and Mean is defined                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.50 (Uwe - 22-April-2001)                            */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> remove PUMA format                                         */
/* -> remove buffering of GRIB reading:                          */
/*    remove: GribFromBuffer, SyncBuffer                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.51 (Uwe - 02-May-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> read news from /pf/m/m214003/doc/afterburner.doc           */
/* -> write stat to  /mf/m/m214003/usage/after                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.52 (Uwe - 04-May-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> set buffersize of inputfile to FILEBUFFERSIZE              */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.53 (Uwe - 31-May-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> Add new Namelist parameter INTERVAL to change the output   */
/*    interval between monthly and daily                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.54 (Uwe - 5-Jun-2001)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> compute GeopotHeight only if it is not deteted             */
/* -> add 1500 byte to igrib buffer in codegb                    */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.55 (Uwe - 10-Jul-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> set FILEBUFFERSIZE to  16384 bytes                         */
/* -> set INTERVAL to UNLIM_INTERVAL to process all timesteps    */
/* -> option -w to write warnings instead of program termination */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.56 (Uwe - 31-Oct-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> T85 and T31 support                                        */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.60 (Uwe - 16-Nov-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> new selection parameter >layer< to select multilevel codes */
/*    which are not on hybrid or pressure level                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.70 (Uwe - 18-Dec-2001)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change GRIB IO to gribex function layer                    */
/* -> change file buffer size to filesystem blocksize            */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.71 (Uwe - 7-Jan-2002)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> clean up                                                   */
/* -> fix problem with dependencies for u and v (call to scaluv )*/
/*    for presure level data type 70                             */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.72 (Uwe - 31-Jan-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> calc pold for type >= 20                                   */
/* -> set level to 0 for code 152 only for leveltype != hybrid   */
/* -> clean up                                                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.73 (Uwe - 15-Feb-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix: calc mean of 131, if only 131 in input            */
/* -> add dmemory to debug memory ( set MEMORY_DEBUG to 1 )      */
/* -> bug fix: found two memory leaks if process more than one   */
/*             timestep                                          */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.74 (Uwe - 05-Mar-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix: EchamDependencies: u+v needed for type >=50       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.75 (Uwe - 11-Mar-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix: set head[1] to 0 only if hlev = 1                 */
/* -> T511 support (MAX lon*lat = 512*1024)                      */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.80 (Uwe - 25-Jul-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* use module file.c for full buffered read of GRIB files.       */
/* new gribex version                                    .       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.81 (Uwe - 1-Aug-2002)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> move decode of complex packed spherical harmonics to       */
/*    gribex_a                                                   */
/* -> optimize decfp2 and confp3                                 */
/* -> add vector pagma for fft on SX                             */
/* -> use trig array of sin and cos for fft                      */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.82 (Uwe - 28-Aug-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> read legendre polynoms from file removed                   */
/* -> replace pointer to struct All with defines                 */
/* -> dynamic allocation of Field and Latitude dependent vars    */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.83 (Uwe - 17-Sep-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> gribex with option "R" to convert reduced grid automaticly */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.90 (Uwe - 18-Sep-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change GRIB Input to GIO library                           */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.91 (Uwe - 7-Oct-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> write variance to 2nd output file                          */
/* -> remove swapping of hybrid level fields                     */
/* -> remove select of layers                                    */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.92 (Uwe - 30-Oct-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> add timestep counter                                       */
/* -> declare all computed variables with MakeDependencies       */
/* -> use getopt for commandline parsing                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.93 (Uwe - 29-Nov-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change output to GIO Library for type 20 and 70            */
/*    use namelist parameter NETCDF=1 to write netCDF files      */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.94 (Uwe - 17-Dec-2002)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> new gio library with mmap to read GRIB file                */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.95 (Uwe - 27-Jan-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> gio library version 0.5.2                                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 3.96 (Uwe - 31-Jan-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change output to GIO library for type 0, 10, 11, 30, 40,   */
/*    41, 50, 60 and 61                                          */
/* -> GIO only version. All GRIB and SERVICE code removed.       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.00 (Uwe - 05-Mar-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> gdilib version 0.5.4 with griblib version 0.4.2            */
/* -> use output precision from input                            */
/* -> new namelist parameter PREC to set output precision        */
/* -> buf fix for TIMESEL                                        */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.01 (Uwe - 19-Mar-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> MakeDependencies: bug fix for windspeed                    */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.02 (Uwe - 28-Mar-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix process files > 4GB                                */
/*    using gdilib 0.5.5, griblib 0.4.3 and filelib 1.3.0        */
/* -> Large File Support (LFS) on linux and sun                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.03 (Uwe - 15-Apr-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> use allways PS_PROG from LNPS to calculate pressure level  */
/* -> bug fix gribex: griblib 0.4.4                              */
/* -> change memory allocation and error handling                */
/*    removed: CharAlloc, IntAlloc and FloatAlloc                */
/*    removed: Abort, Stars, NewLine                             */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.04 (Uwe - 26-Jul-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> using gdilib 0.5.9, griblib 0.4.5 and filelib 1.3.1        */
/* (-> new computation of code 263 for ECHAM5 data (canceled))   */
/* -> no computation of code 261-264 for ECHAM5 data             */
/* -> parini: bugfix for level=-1 and level=0                    */
/* -> procstat: set logfile to /pf/m/m214003/local/log/after.log */
/* -> Control: bugfix set overtID if ivertID == code.ivertID     */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.05 (Uwe - 26-Aug-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> gdilib: bug fix ( netCDF basetime minute was always 0)     */
/* -> defineLevel: bug fix for files with two hybrid level       */
/*                definition (e.g. ECMWF code129 1 hybrid level) */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.10 (Uwe - 16-Sep-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> using cdilib 0.7.0 and griblib 0.5.0                       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.11 (Uwe -  5-Oct-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> changes to vlist                                           */
/* -> using cdilib 0.7.3                                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.12 (Uwe -  5-Nov-2003)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> using cdilib 0.7.6                                         */
/* -> speedup fft (sx: nfft=1024 else: nfft=64)                  */
/* -> change debug option form -v to -d                          */
/* -> new option -v vct to read optional vct from file           */
/* -> using default vct for L191 if not present (VCT_L191)       */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.13 (Uwe -  20-Nov-2003)                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> read unlimited inputfiles from the commandline             */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.14 (Uwe -  18-Dec-2003)                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> OpenMP test version                                        */
/* -> using cdilib 0.7.7                                         */
/* -> bug fix for different hybrid levels (Noel)                 */
/* -> speed up phcs on SX-6 (60%) by removing unnecessary        */
/*    novector pragma                                            */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.15 (Uwe -  29-Jun-2004)                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> SwitchFile: Bug fix for MULTI with relative path name      */
/* -> set min I/O Buffersize to 128KB                            */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.16 (Uwe -  10-Sep-2004)                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> make dependencies for slp from geopotential                */
/* -> clean up vertical interpolation for height level           */
/*    - h2p: change phlev from 3D to 1D                          */
/*    - geninz: removed (now use genind)                         */
/*    - Interpolate_X_Z: removed (now use interp_X)              */
/*    - Interpolate_T_Z: removed (now use interp_T)              */
/*    - Interpolate_Z_Z: removed (now use interp_Z)              */
/*                                                               */
/* -> new option -m to use missing vales for undefined gridpoints*/
/*    - used in interp_X, interp_T and interp_Z                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.17 (Uwe - 27-Oct-2004)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 0.8.4                        */
/* -> FORMAT=4 for netCDF2                                       */
/* -> change option -m to namelist parameter EXTRAPOLATE         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.18 (Uwe - 21-Feb-2005)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 0.8.8                        */
/* -> bug fix: compute PS_PROG only if .comp flag is set         */
/* -> missing value support                                      */
/*    - for MEAN=0 or MEAN=1 and TYPE=20/30/70                   */
/*    - with EXTRAPOLATE=0: MEAN=0 and TYPE=30                   */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.20 (Uwe - 4-Mar-2005)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 0.9.1                        */
/* -> Parallel reading                                           */
/*          on with option -p                                    */
/*         off with option -P  (default)                         */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.21 (Uwe - 9-Jan-2006)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change MAXLEVEL from 256 to 1024                           */
/* -> change namelist size from 1024 to 65536                    */
/* -> fourier: made ifax and trig local                          */
/* -> bug fix for unitsel                                        */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.22 (Uwe - 1-Feb-2006)                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 0.9.5                        */
/* -> CheckContent every timestep                                */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.23 (Uwe - 11-Apr-2006)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix for select code 36 (HIH_CLOUD) only                */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.30 (Uwe - 18-Sep-2006)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 1.0.2                        */
/* -> add SZIP support                                           */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.31 (Uwe - 15-Feb-2007)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix for codes > MaxCodes (report: Silvia Kloster)      */
/* -> bug fix for nml parameter TIMESEL (report: Falk Niehrster)*/
/* -> set MaxHours from 4 to 24                                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.32 (Uwe - 11-Jun-2007)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> bug fix TYPE=0 and nlev < ml output (report: Rita Seifert) */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.40 (Uwe -  8-Jan-2008)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 1.1.0                        */
/* -> output years with century                                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.41 (Uwe - 21-Oct-2008)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> cdilib: set MAX_ZAXIS to 4096 (bug fix)                    */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/* Version 4.50 (Uwe -  5-May-2009)                              */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 1.3.1                        */
/* -> CheckDependencies: create static ZAXIS_SURFACE (bug fix)   */
/* -> interp_Z: Abort if temperature is missing                  */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
//* Version 4.51 (Uwe - 11-Jun-2009)                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                               */
/* -> change to cdi library version 1.3.2                        */
/* -> added support for netCDF4 format (format = 6)              */
/* -> set default calendar to PROLEPTIC                          */
/*                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
